/* Copyright (C) 1991, 1996, 1998 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>

/* Flush STREAM's buffer.
   If STREAM is NULL, flush the buffers of all streams that are writing.  */
int
fflush (stream)
     register FILE *stream;
{
  if (stream == NULL)
    {
      int lossage = 0;
      for (stream = __stdio_head; stream != NULL; stream = stream->__next)
	if (__validfp (stream) && stream->__mode.__write)
	  lossage |= fflush (stream) == EOF;
      return lossage ? EOF : 0;
    }

  if (!__validfp (stream) || !stream->__mode.__write)
    {
      __set_errno (EINVAL);
      return EOF;
    }

  return __flshfp (stream, EOF);
}

weak_alias(fflush, fflush_unlocked)
