#ifndef __GNU_LIB_NAMES_H
#define __GNU_LIB_NAMES_H	1

#define	LIBBROKENLOCALE_SO	"libBrokenLocale.a"
#define	LIBCRYPT_SO	"libcrypt.a"
#define	LIBC_SO	"libc.a"
#define	LIBDB1_SO	"libdb1.a"
#define	LIBDB_SO	"libdb.a"
#define	LIBDL_SO	"libdl.a"
#define	LIBM_SO	"libm.a"
#define	LIBNOVERSION_SO	"libNoVersion.a"
#define	LIBNSL_SO	"libnsl.a"
#define	LIBNSS1_COMPAT_SO	"libnss1_compat.a"
#define	LIBNSS1_DB_SO	"libnss1_db.a"
#define	LIBNSS1_DNS_SO	"libnss1_dns.a"
#define	LIBNSS1_FILES_SO	"libnss1_files.a"
#define	LIBNSS1_NIS_SO	"libnss1_nis.a"
#define	LIBNSS_COMPAT_SO	"libnss_compat.a"
#define	LIBNSS_DB_SO	"libnss_db.a"
#define	LIBNSS_DNS_SO	"libnss_dns.a"
#define	LIBNSS_FILES_SO	"libnss_files.a"
#define	LIBNSS_HESIOD_SO	"libnss_hesiod.a"
#define	LIBNSS_LDAP_SO	"libnss_ldap.a"
#define	LIBNSS_NISPLUS_SO	"libnss_nisplus.a"
#define	LIBNSS_NIS_SO	"libnss_nis.a"
#define	LIBPTHREAD_SO	"libpthread.a"
#define	LIBRESOLV_SO	"libresolv.a"
#define	LIBRT_SO	"librt.a"
#define	LIBUTIL_SO	"libutil.a"

#endif	/* gnu/lib-names.h */
