/* This file should provide inline versions of math functions.

   Surround GCC-specific parts with #ifdef __GNUC__, and use `extern __inline'.

   This file should define __MATH_INLINES if functions are actually defined as
   inlines.  */

#if !defined __NO_MATH_INLINES && defined __OPTIMIZE__

/* Here goes the real code.  */

#endif
