/* ftruncate for SVR4 using the fcntl F_FREESP command.
   Copyright (C) 1995, 1997 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */

#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>

/* Truncate the file FD refers to to LENGTH bytes.  */
int
ftruncate (fd, length)
     int fd;
     off_t length;
{
  struct flock fl;

  memset (&fl, 0, sizeof fl);
  fl.l_type = F_WRLCK;
  fl.l_start = length;
  return fcntl (fd, F_FREESP, &fl);
}
