/*
 * Defines the country codes and abbreviations according to ISO 3166.
 * This is used in ld-address.c (address_finish).
 *
 * If you find something missing or wrong contact <bug-glibc@gnu.org>
 */
DEFINE_COUNTRY_CODE ("AFGHANISTAN", AF, AFG, 4)
DEFINE_COUNTRY_CODE ("ALBANIA", AL, ALB, 8)
DEFINE_COUNTRY_CODE ("ALGERIA", DZ, DZA, 12)
DEFINE_COUNTRY_CODE ("AMERICAN SAMOA", AS, ASM, 16)
DEFINE_COUNTRY_CODE ("ANDORRA", AD, AND, 20)
DEFINE_COUNTRY_CODE ("ANGOLA", AO, AGO, 24)
DEFINE_COUNTRY_CODE ("ANGUILLA", AI, AIA, 660)
DEFINE_COUNTRY_CODE ("ANTARCTICA", AQ, ATA, 10)
DEFINE_COUNTRY_CODE ("ANTIGUA AND BARBUDA", AG, ATG, 28)
DEFINE_COUNTRY_CODE ("ARGENTINA", AR, ARG, 32)
DEFINE_COUNTRY_CODE ("ARMENIA", AM, ARM, 51)
DEFINE_COUNTRY_CODE ("ARUBA", AW, ABW, 533)
DEFINE_COUNTRY_CODE ("AUSTRALIA", AU, AUS, 36)
DEFINE_COUNTRY_CODE ("AUSTRIA", AT, AUT, 40)
DEFINE_COUNTRY_CODE ("AZERBAIJAN", AZ, AZE, 31)
DEFINE_COUNTRY_CODE ("BAHAMAS", BS, BHS, 44)
DEFINE_COUNTRY_CODE ("BAHRAIN", BH, BHR, 48)
DEFINE_COUNTRY_CODE ("BANGLADESH", BD, BGD, 50)
DEFINE_COUNTRY_CODE ("BARBADOS", BB, BRB, 52)
DEFINE_COUNTRY_CODE ("BELARUS", BY, BLR, 112)
DEFINE_COUNTRY_CODE ("BELGIUM", BE, BEL, 56)
DEFINE_COUNTRY_CODE ("BELIZE", BZ, BLZ, 84)
DEFINE_COUNTRY_CODE ("BENIN", BJ, BEN, 204)
DEFINE_COUNTRY_CODE ("BERMUDA", BM, BMU, 60)
DEFINE_COUNTRY_CODE ("BHUTAN", BT, BTN, 64)
DEFINE_COUNTRY_CODE ("BOLIVIA", BO, BOL, 68)
DEFINE_COUNTRY_CODE ("BOSNIA AND HERZEGOVINA", BA, BIH, 70)
DEFINE_COUNTRY_CODE ("BOTSWANA", BW, BWA, 72)
DEFINE_COUNTRY_CODE ("BOUVET ISLAND", BV, BVT, 74)
DEFINE_COUNTRY_CODE ("BRAZIL", BR, BRA, 76)
DEFINE_COUNTRY_CODE ("BRITISH INDIAN OCEAN TERRITORY", IO, IOT, 86)
DEFINE_COUNTRY_CODE ("BRUNEI DARUSSALAM", BN, BRN, 96)
DEFINE_COUNTRY_CODE ("BULGARIA", BG, BGR, 100)
DEFINE_COUNTRY_CODE ("BURKINA FASO", BF, BFA, 854)
DEFINE_COUNTRY_CODE ("BURUNDI", BI, BDI, 108)
DEFINE_COUNTRY_CODE ("CAMBODIA", KH, KHM, 116)
DEFINE_COUNTRY_CODE ("CAMEROON", CM, CMR, 120)
DEFINE_COUNTRY_CODE ("CANADA", CA, CAN, 124)
DEFINE_COUNTRY_CODE ("CAPE VERDE", CV, CPV, 132)
DEFINE_COUNTRY_CODE ("CAYMAN ISLANDS", KY, CYM, 136)
DEFINE_COUNTRY_CODE ("CENTRAL AFRICAN REPUBLIC", CF, CAF, 140)
DEFINE_COUNTRY_CODE ("CHAD", TD, TCD, 148)
DEFINE_COUNTRY_CODE ("CHILE", CL, CHL, 152)
DEFINE_COUNTRY_CODE ("CHINA", CN, CHN, 156)
DEFINE_COUNTRY_CODE ("CHRISTMAS ISLAND", CX, CXR, 162)
DEFINE_COUNTRY_CODE ("COCOS (KEELING) ISLANDS", CC, CCK, 166)
DEFINE_COUNTRY_CODE ("COLOMBIA", CO, COL, 170)
DEFINE_COUNTRY_CODE ("COMOROS", KM, COM, 174)
DEFINE_COUNTRY_CODE ("CONGO", CG, COG, 178)
DEFINE_COUNTRY_CODE ("CONGO, THE DEMOCRATIC REPUBLIC OF THE", CD, COD, 180)
DEFINE_COUNTRY_CODE ("COOK ISLANDS", CK, COK, 184)
DEFINE_COUNTRY_CODE ("COSTA RICA", CR, CRI, 188)
DEFINE_COUNTRY_CODE ("COTE D'IVOIRE", CI, CIV, 384)
DEFINE_COUNTRY_CODE ("CROATIA", HR, HRV, 191)
DEFINE_COUNTRY_CODE ("CUBA", CU, CUB, 192)
DEFINE_COUNTRY_CODE ("CYPRUS", CY, CYP, 196)
DEFINE_COUNTRY_CODE ("CZECH REPUBLIC", CZ, CZE, 203)
DEFINE_COUNTRY_CODE ("DENMARK", DK, DNK, 208)
DEFINE_COUNTRY_CODE ("DJIBOUTI", DJ, DJI, 262)
DEFINE_COUNTRY_CODE ("DOMINICA", DM, DMA, 212)
DEFINE_COUNTRY_CODE ("DOMINICAN REPUBLIC", DO, DOM, 214)
DEFINE_COUNTRY_CODE ("EAST TIMOR", TP, TMP, 626)
DEFINE_COUNTRY_CODE ("ECUADOR", EC, ECU, 218)
DEFINE_COUNTRY_CODE ("EGYPT", EG, EGY, 818)
DEFINE_COUNTRY_CODE ("EL SALVADOR", SV, SLV, 222)
DEFINE_COUNTRY_CODE ("EQUATORIAL GUINEA", GQ, GNQ, 226)
DEFINE_COUNTRY_CODE ("ERITREA", ER, ERI, 232)
DEFINE_COUNTRY_CODE ("ESTONIA", EE, EST, 233)
DEFINE_COUNTRY_CODE ("ETHIOPIA", ET, ETH, 231)
DEFINE_COUNTRY_CODE ("FALKLAND ISLANDS (MALVINAS)", FK, FLK, 238)
DEFINE_COUNTRY_CODE ("FAROE ISLANDS", FO, FRO, 234)
DEFINE_COUNTRY_CODE ("FIJI", FJ, FJI, 242)
DEFINE_COUNTRY_CODE ("FINLAND", FI, FIN, 246)
DEFINE_COUNTRY_CODE ("FRANCE", FR, FRA, 250)
DEFINE_COUNTRY_CODE ("FRENCH GUIANA", GF, GUF, 254)
DEFINE_COUNTRY_CODE ("FRENCH POLYNESIA", PF, PYF, 258)
DEFINE_COUNTRY_CODE ("FRENCH SOUTHERN TERRITORIES", TF, ATF, 260)
DEFINE_COUNTRY_CODE ("GABON", GA, GAB, 266)
DEFINE_COUNTRY_CODE ("GAMBIA", GM, GMB, 270)
DEFINE_COUNTRY_CODE ("GEORGIA", GE, GEO, 268)
DEFINE_COUNTRY_CODE ("GERMANY", DE, DEU, 276)
DEFINE_COUNTRY_CODE ("GHANA", GH, GHA, 288)
DEFINE_COUNTRY_CODE ("GIBRALTAR", GI, GIB, 292)
DEFINE_COUNTRY_CODE ("GREECE", GR, GRC, 300)
DEFINE_COUNTRY_CODE ("GREENLAND", GL, GRL, 304)
DEFINE_COUNTRY_CODE ("GRENADA", GD, GRD, 308)
DEFINE_COUNTRY_CODE ("GUADELOUPE", GP, GLP, 312)
DEFINE_COUNTRY_CODE ("GUAM", GU, GUM, 316)
DEFINE_COUNTRY_CODE ("GUATEMALA", GT, GTM, 320)
DEFINE_COUNTRY_CODE ("GUINEA", GN, GIN, 324)
DEFINE_COUNTRY_CODE ("GUINEA-BISSAU", GW, GNB, 624)
DEFINE_COUNTRY_CODE ("GUYANA", GY, GUY, 328)
DEFINE_COUNTRY_CODE ("HAITI", HT, HTI, 332)
DEFINE_COUNTRY_CODE ("HEARD ISLAND AND MCDONALD ISLANDS", HM, HMD, 334)
DEFINE_COUNTRY_CODE ("HOLY SEE (VATICAN CITY STATE)", VA, VAT, 336)
DEFINE_COUNTRY_CODE ("HONDURAS", HN, HND, 340)
DEFINE_COUNTRY_CODE ("HONG KONG", HK, HKG, 344)
DEFINE_COUNTRY_CODE ("HUNGARY", HU, HUN, 348)
DEFINE_COUNTRY_CODE ("ICELAND", IS, ISL, 352)
DEFINE_COUNTRY_CODE ("INDIA", IN, IND, 356)
DEFINE_COUNTRY_CODE ("INDONESIA", ID, IDN, 360)
DEFINE_COUNTRY_CODE ("IRAN, ISLAMIC REPUBLIC OF", IR, IRN, 364)
DEFINE_COUNTRY_CODE ("IRAQ", IQ, IRQ, 368)
DEFINE_COUNTRY_CODE ("IRELAND", IE, IRL, 372)
DEFINE_COUNTRY_CODE ("ISRAEL", IL, ISR, 376)
DEFINE_COUNTRY_CODE ("ITALY", IT, ITA, 380)
DEFINE_COUNTRY_CODE ("JAMAICA", JM, JAM, 388)
DEFINE_COUNTRY_CODE ("JAPAN", JP, JPN, 392)
DEFINE_COUNTRY_CODE ("JORDAN", JO, JOR, 400)
DEFINE_COUNTRY_CODE ("KAZAKSTAN", KZ, KAZ, 398)
DEFINE_COUNTRY_CODE ("KENYA", KE, KEN, 404)
DEFINE_COUNTRY_CODE ("KIRIBATI", KI, KIR, 296)
DEFINE_COUNTRY_CODE ("KOREA, DEMOCRATIC PEOPLE'S REPUBLIC OF", KP, PRK, 408)
DEFINE_COUNTRY_CODE ("KOREA, REPUBLIC OF", KR, KOR, 410)
DEFINE_COUNTRY_CODE ("KUWAIT", KW, KWT, 414)
DEFINE_COUNTRY_CODE ("KYRGYZSTAN", KG, KGZ, 417)
DEFINE_COUNTRY_CODE ("LAO PEOPLE'S DEMOCRATIC REPUBLIC", LA, LAO, 418)
DEFINE_COUNTRY_CODE ("LATVIA", LV, LVA, 428)
DEFINE_COUNTRY_CODE ("LEBANON", LB, LBN, 422)
DEFINE_COUNTRY_CODE ("LESOTHO", LS, LSO, 426)
DEFINE_COUNTRY_CODE ("LIBERIA", LR, LBR, 430)
DEFINE_COUNTRY_CODE ("LIBYAN ARAB JAMAHIRIYA", LY, LBY, 434)
DEFINE_COUNTRY_CODE ("LIECHTENSTEIN", LI, LIE, 438)
DEFINE_COUNTRY_CODE ("LITHUANIA", LT, LTU, 440)
DEFINE_COUNTRY_CODE ("LUXEMBOURG", LU, LUX, 442)
DEFINE_COUNTRY_CODE ("MACAU", MO, MAC, 446)
DEFINE_COUNTRY_CODE ("MACEDONIA, THE FORMER YUGOSLAV REPUBLIC OF", MK, MKD, 807)
DEFINE_COUNTRY_CODE ("MADAGASCAR", MG, MDG, 450)
DEFINE_COUNTRY_CODE ("MALAWI", MW, MWI, 454)
DEFINE_COUNTRY_CODE ("MALAYSIA", MY, MYS, 458)
DEFINE_COUNTRY_CODE ("MALDIVES", MV, MDV, 462)
DEFINE_COUNTRY_CODE ("MALI", ML, MLI, 466)
DEFINE_COUNTRY_CODE ("MALTA", MT, MLT, 470)
DEFINE_COUNTRY_CODE ("MARSHALL ISLANDS", MH, MHL, 584)
DEFINE_COUNTRY_CODE ("MARTINIQUE", MQ, MTQ, 474)
DEFINE_COUNTRY_CODE ("MAURITANIA", MR, MRT, 478)
DEFINE_COUNTRY_CODE ("MAURITIUS", MU, MUS, 480)
DEFINE_COUNTRY_CODE ("MAYOTTE", YT, MYT, 175)
DEFINE_COUNTRY_CODE ("MEXICO", MX, MEX, 484)
DEFINE_COUNTRY_CODE ("MICRONESIA, FEDERATED STATES OF", FM, FSM, 583)
DEFINE_COUNTRY_CODE ("MOLDOVA, REPUBLIC OF", MD, MDA, 498)
DEFINE_COUNTRY_CODE ("MONACO", MC, MCO, 492)
DEFINE_COUNTRY_CODE ("MONGOLIA", MN, MNG, 496)
DEFINE_COUNTRY_CODE ("MONTSERRAT", MS, MSR, 500)
DEFINE_COUNTRY_CODE ("MOROCCO", MA, MAR, 504)
DEFINE_COUNTRY_CODE ("MOZAMBIQUE", MZ, MOZ, 508)
DEFINE_COUNTRY_CODE ("MYANMAR", MM, MMR, 104)
DEFINE_COUNTRY_CODE ("NAMIBIA", NA, NAM, 516)
DEFINE_COUNTRY_CODE ("NAURU", NR, NRU, 520)
DEFINE_COUNTRY_CODE ("NEPAL", NP, NPL, 524)
DEFINE_COUNTRY_CODE ("NETHERLANDS", NL, NLD, 528)
DEFINE_COUNTRY_CODE ("NETHERLANDS ANTILLES", AN, ANT, 530)
DEFINE_COUNTRY_CODE ("NEW CALEDONIA", NC, NCL, 540)
DEFINE_COUNTRY_CODE ("NEW ZEALAND", NZ, NZL, 554)
DEFINE_COUNTRY_CODE ("NICARAGUA", NI, NIC, 558)
DEFINE_COUNTRY_CODE ("NIGER", NE, NER, 562)
DEFINE_COUNTRY_CODE ("NIGERIA", NG, NGA, 566)
DEFINE_COUNTRY_CODE ("NIUE", NU, NIU, 570)
DEFINE_COUNTRY_CODE ("NORFOLK ISLAND", NF, NFK, 574)
DEFINE_COUNTRY_CODE ("NORTHERN MARIANA ISLANDS", MP, MNP, 580)
DEFINE_COUNTRY_CODE ("NORWAY", NO, NOR, 578)
DEFINE_COUNTRY_CODE ("OMAN", OM, OMN, 512)
DEFINE_COUNTRY_CODE ("PAKISTAN", PK, PAK, 586)
DEFINE_COUNTRY_CODE ("PALAU", PW, PLW, 585)
DEFINE_COUNTRY_CODE ("PALESTINIAN TERRITORY, OCCUPIED", PS, PSE, 275)
DEFINE_COUNTRY_CODE ("PANAMA", PA, PAN, 591)
DEFINE_COUNTRY_CODE ("PAPUA NEW GUINEA", PG, PNG, 598)
DEFINE_COUNTRY_CODE ("PARAGUAY", PY, PRY, 600)
DEFINE_COUNTRY_CODE ("PERU", PE, PER, 604)
DEFINE_COUNTRY_CODE ("PHILIPPINES", PH, PHL, 608)
DEFINE_COUNTRY_CODE ("PITCAIRN", PN, PCN, 612)
DEFINE_COUNTRY_CODE ("POLAND", PL, POL, 616)
DEFINE_COUNTRY_CODE ("PORTUGAL", PT, PRT, 620)
DEFINE_COUNTRY_CODE ("PUERTO RICO", PR, PRI, 630)
DEFINE_COUNTRY_CODE ("QATAR", QA, QAT, 634)
DEFINE_COUNTRY_CODE ("REUNION", RE, REU, 638)
DEFINE_COUNTRY_CODE ("ROMANIA", RO, ROM, 642)
DEFINE_COUNTRY_CODE ("RUSSIAN FEDERATION", RU, RUS, 643)
DEFINE_COUNTRY_CODE ("RWANDA", RW, RWA, 646)
DEFINE_COUNTRY_CODE ("SAINT HELENA", SH, SHN, 654)
DEFINE_COUNTRY_CODE ("SAINT KITTS AND NEVIS", KN, KNA, 659)
DEFINE_COUNTRY_CODE ("SAINT LUCIA", LC, LCA, 662)
DEFINE_COUNTRY_CODE ("SAINT PIERRE AND MIQUELON", PM, SPM, 666)
DEFINE_COUNTRY_CODE ("SAINT VINCENT AND THE GRENADINES", VC, VCT, 670)
DEFINE_COUNTRY_CODE ("SAMOA", WS, WSM, 882)
DEFINE_COUNTRY_CODE ("SAN MARINO", SM, SMR, 674)
DEFINE_COUNTRY_CODE ("SAO TOME AND PRINCIPE", ST, STP, 678)
DEFINE_COUNTRY_CODE ("SAUDI ARABIA", SA, SAU, 682)
DEFINE_COUNTRY_CODE ("SENEGAL", SN, SEN, 686)
DEFINE_COUNTRY_CODE ("SEYCHELLES", SC, SYC, 690)
DEFINE_COUNTRY_CODE ("SIERRA LEONE", SL, SLE, 694)
DEFINE_COUNTRY_CODE ("SINGAPORE", SG, SGP, 702)
DEFINE_COUNTRY_CODE ("SLOVAKIA", SK, SVK, 703)
DEFINE_COUNTRY_CODE ("SLOVENIA", SI, SVN, 705)
DEFINE_COUNTRY_CODE ("SOLOMON ISLANDS", SB, SLB, 90)
DEFINE_COUNTRY_CODE ("SOMALIA", SO, SOM, 706)
DEFINE_COUNTRY_CODE ("SOUTH AFRICA", ZA, ZAF, 710)
DEFINE_COUNTRY_CODE ("SOUTH GEORGIA AND THE SOUTH SANDWICH ISLANDS", GS, SGS, 239)
DEFINE_COUNTRY_CODE ("SPAIN", ES, ESP, 724)
DEFINE_COUNTRY_CODE ("SRI LANKA", LK, LKA, 144)
DEFINE_COUNTRY_CODE ("SUDAN", SD, SDN, 736)
DEFINE_COUNTRY_CODE ("SURINAME", SR, SUR, 740)
DEFINE_COUNTRY_CODE ("SVALBARD AND JAN MAYEN", SJ, SJM, 744)
DEFINE_COUNTRY_CODE ("SWAZILAND", SZ, SWZ, 748)
DEFINE_COUNTRY_CODE ("SWEDEN", SE, SWE, 752)
DEFINE_COUNTRY_CODE ("SWITZERLAND", CH, CHE, 756)
DEFINE_COUNTRY_CODE ("SYRIAN ARAB REPUBLIC", SY, SYR, 760)
DEFINE_COUNTRY_CODE ("TAIWAN, PROVINCE OF CHINA", TW, TWN, 158)
DEFINE_COUNTRY_CODE ("TAJIKISTAN", TJ, TJK, 762)
DEFINE_COUNTRY_CODE ("TANZANIA, UNITED REPUBLIC OF", TZ, TZA, 834)
DEFINE_COUNTRY_CODE ("THAILAND", TH, THA, 764)
DEFINE_COUNTRY_CODE ("TOGO", TG, TGO, 768)
DEFINE_COUNTRY_CODE ("TOKELAU", TK, TKL, 772)
DEFINE_COUNTRY_CODE ("TONGA", TO, TON, 776)
DEFINE_COUNTRY_CODE ("TRINIDAD AND TOBAGO", TT, TTO, 780)
DEFINE_COUNTRY_CODE ("TUNISIA", TN, TUN, 788)
DEFINE_COUNTRY_CODE ("TURKEY", TR, TUR, 792)
DEFINE_COUNTRY_CODE ("TURKMENISTAN", TM, TKM, 795)
DEFINE_COUNTRY_CODE ("TURKS AND CAICOS ISLANDS", TC, TCA, 796)
DEFINE_COUNTRY_CODE ("TUVALU", TV, TUV, 798)
DEFINE_COUNTRY_CODE ("UGANDA", UG, UGA, 800)
DEFINE_COUNTRY_CODE ("UKRAINE", UA, UKR, 804)
DEFINE_COUNTRY_CODE ("UNITED ARAB EMIRATES", AE, ARE, 784)
DEFINE_COUNTRY_CODE ("UNITED KINGDOM", GB, GBR, 826)
DEFINE_COUNTRY_CODE ("UNITED STATES", US, USA, 840)
DEFINE_COUNTRY_CODE ("UNITED STATES MINOR OUTLYING ISLANDS", UM, UMI, 581)
DEFINE_COUNTRY_CODE ("URUGUAY", UY, URY, 858)
DEFINE_COUNTRY_CODE ("UZBEKISTAN", UZ, UZB, 860)
DEFINE_COUNTRY_CODE ("VANUATU", VU, VUT, 548)
DEFINE_COUNTRY_CODE ("VENEZUELA", VE, VEN, 862)
DEFINE_COUNTRY_CODE ("VIET NAM", VN, VNM, 704)
DEFINE_COUNTRY_CODE ("VIRGIN ISLANDS, BRITISH", VG, VGB, 92)
DEFINE_COUNTRY_CODE ("VIRGIN ISLANDS, U.S.", VI, VIR, 850)
DEFINE_COUNTRY_CODE ("WALLIS AND FUTUNA", WF, WLF, 876)
DEFINE_COUNTRY_CODE ("WESTERN SAHARA", EH, ESH, 732)
DEFINE_COUNTRY_CODE ("YEMEN", YE, YEM, 887)
DEFINE_COUNTRY_CODE ("YUGOSLAVIA", YU, YUG, 891)
DEFINE_COUNTRY_CODE ("ZAMBIA", ZM, ZMB, 894)
DEFINE_COUNTRY_CODE ("ZIMBABWE", ZW, ZWE, 716)
